{%- if theme_logo %}
<p class="logo">
 <a href="{{ theme_website }}">
  <img class="logo" src="{{ pathto('_static/' ~ theme_logo, 1) }}" alt="Logo"/>
 </a>
{%- if theme_logo_caption %}
<br/ >{{ theme_logo_caption }}
{%- endif %}

</p>
{%- endif %}

<h3><a href="{{ pathto(master_doc) }}">Manual</a></h3>
{{ toctree() }}

{%- if theme_wiki %}
<ul>
 <li class="toctree-l1">
 <a class="reference external" href="{{ theme_wiki }}">Wiki</a>
 </li>
</ul>
{%- endif %}

{%- if pagename != "search" and builder != "singlehtml" %}
<div id="searchbox" role="search">
    <form class="search" action="{{ pathto('search') }}" method="get">
      <input type="text" name="q" size="12" />
      <input type="submit" value="Search" />
      <input type="hidden" name="check_keywords" value="yes" />
      <input type="hidden" name="area" value="default" />
    </form>
</div>
{%- endif %}
