// This file is part of fityk program. Copyright 2009 Marcin Wojdyr
// Licence: GNU General Public License ver. 2+
// AboutDlg: "about..." dialog

#ifndef FITYK_WX_ABOUT_H_
#define FITYK_WX_ABOUT_H_

#include <wx/wx.h>


class AboutDlg : public wxDialog
{
public:
    AboutDlg(wxWindow* parent);
};

#endif // FITYK_WX_ABOUT_H_
